import 'package:flutter/material.dart';
import 'package:contacts_service/contacts_service.dart';
import 'package:permission_handler/permission_handler.dart';

class GetContectNumber extends StatefulWidget {
  const GetContectNumber({
    Key? key,
    this.builder,
    this.loadingBuilder,
    required this.onSelect,
  }) : super(key: key);
  final Widget Function(BuildContext context, void Function() selectContent,
      ContactInfo? lastSelected)? builder;
  final Widget Function(
    BuildContext context,
  )? loadingBuilder;

  final void Function(ContactInfo contact) onSelect;

  @override
  State<GetContectNumber> createState() => _GetContectNumberState();
}

class _GetContectNumberState extends State<GetContectNumber> {
  var loading = false;
  ContactInfo? lastSelected;

  @override
  Widget build(BuildContext context) {
    if (loading) {
      return widget.loadingBuilder?.call(context) ??
          const Padding(
            padding: EdgeInsets.symmetric(vertical: 28, horizontal: 8),
            child: LinearProgressIndicator(),
          );
    }
    return widget.builder?.call(context, onPressed, lastSelected) ??
        Padding(
          padding: const EdgeInsets.all(8.0),
          child: ElevatedButton.icon(
            onPressed: onPressed,
            icon: const Icon(Icons.contacts_rounded),
            label: const Text("From Contact"),
          ),
        );
  }

  void onPressed() async {
    if (loading) return;
    setState(() {
      loading = true;
    });
    final permissionStatus = await _getPermission();
    if (permissionStatus == PermissionStatus.granted) {
      final contacts = await ContactsService.getContacts(withThumbnails: false);
      if (!mounted) return;
      final info =
          await Navigator.push(context, MaterialPageRoute(builder: (context) {
        return _SelectContact(contacts: contacts);
      }));
      if (info is ContactInfo) {
        widget.onSelect(info);
        lastSelected = info;
      }
    } else {
      showDialog(
        context: context,
        builder: (BuildContext context) => AlertDialog(
          title: const Text('Permissions error'),
          content: const Text('Please enable contacts access '
              'permission in system settings'),
          actions: [
            TextButton(
              child: const Text('OK'),
              onPressed: () => Navigator.of(context).pop(),
            )
          ],
        ),
      );
    }
    setState(() {
      loading = false;
    });
  }

  Future<PermissionStatus> _getPermission() async {
    final permission = await Permission.contacts.status;
    if (permission != PermissionStatus.granted) {
      final permissionStatus = await [Permission.contacts].request();
      return permissionStatus[Permission.contacts] ?? PermissionStatus.limited;
    } else {
      return permission;
    }
  }
}

class ContactInfo {
  final String phoneNumber;
  final String? name;

  ContactInfo(this.name, this.phoneNumber);

  @override
  String toString() {
    return {name, phoneNumber}.toString();
  }
}

class _SelectContact extends StatefulWidget {
  const _SelectContact({Key? key, required this.contacts}) : super(key: key);
  final List<Contact> contacts;

  @override
  State<_SelectContact> createState() => _SelectContactState();
}

class _SelectContactState extends State<_SelectContact> {
  List<ContactInfo> get phoneNumbers {
    final result = <ContactInfo>[];
    final numbers = <String>{};
    for (var contact in widget.contacts) {
      final phones = contact.phones;
      if (phones == null) continue;
      for (var item in phones) {
        var number = item.value ?? "";
        if (number.isEmpty) continue;
        if (number.startsWith("+91")) number = number.substring(3);
        if (number.contains("+")) continue;
        if (number.startsWith("0")) number = number.substring(1);
        number = number.replaceAll("-", "").replaceAll(" ", "");
        if (number.length != 10) continue;
        if (numbers.add(number)) {
          result.add(ContactInfo(contact.displayName, number));
        }
      }
    }
    return result;
  }

  @override
  Widget build(BuildContext context) {
    final numbers = phoneNumbers;
    return Scaffold(
      appBar: AppBar(
        title: const Text("Select Contact"),
        actions: [
          IconButton(
            onPressed: () async {
              final info = await showSearch(
                context: context,
                delegate: _Search(numbers),
              );
              if (!mounted) return;
              if (info != null) Navigator.pop(context, info);
            },
            icon: const Icon(Icons.search),
          )
        ],
      ),
      body: ListView.builder(
        itemCount: numbers.length,
        itemBuilder: (context, index) {
          final info = numbers.elementAt(index);
          return ListTile(
            title: Text(info.name ?? "-- * --"),
            trailing: Text(info.phoneNumber),
            onTap: () {
              Navigator.pop(context, info);
            },
          );
        },
      ),
    );
  }
}

class _Search extends SearchDelegate<ContactInfo?> {
  final List<ContactInfo> contacts;

  _Search(this.contacts) : super();

  @override
  List<Widget>? buildActions(BuildContext context) {
    return [
      IconButton(
        onPressed: () {
          if (query.isEmpty) {
            close(context, null);
          } else {
            query = "";
          }
        },
        icon: const Icon(Icons.clear),
      )
    ];
  }

  @override
  Widget? buildLeading(BuildContext context) {
    return null;
  }

  @override
  Widget buildResults(BuildContext context) {
    return buildSuggestions(context);
  }

  @override
  Widget buildSuggestions(BuildContext context) {
    if (query.isEmpty) {
      return Padding(
        padding: const EdgeInsets.symmetric(horizontal: 8.0, vertical: 100),
        child: Text(
          "Type Person`s Name To Filter Contacts",
          style: Theme.of(context).textTheme.headline4,
        ),
      );
    }
    final suggestions = contacts.where(
      (info) => info.name?.toLowerCase().contains(query.toLowerCase()) ?? false,
    );
    return ListView.builder(
      itemCount: suggestions.length,
      itemBuilder: (context, index) {
        final info = suggestions.elementAt(index);
        return ListTile(
          title: Text(info.name ?? "-- * --"),
          trailing: Text(info.phoneNumber),
          onTap: () {
            close(context, info);
          },
        );
      },
    );
  }
}
